/**<pre>
 * Title: GPS Simulator
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package app.navigator.poi;

import app.navigator.util.*;

import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import org.w3c.dom.*;
import java.io.*;
import java.util.*;
import java.awt.*;



/* ====================================================================== */
/** This class reads in a given XML file and parses it for a variety of
 *  values which it will store in its private variables.  It then provides
 *  methods for retrieving the values.
 *
 */
public class POIParser{


    /** The directory where the configuration files are located. */
    public static final String DIRECTORY = "files/app/data/";

	/** Our vector of reference points */
    private Vector pois = new Vector();

	/** The XML doc we create from given filename. */
    private DocumentBuilder db = null;



    /* ===================================================================== */
    /** This constructor builds up an XML Document of Nodes from the given XML
     *  file and then calls the method echo to retrieve the values of certain
     *  nodes.
     *
     *  @param  filename   The filename of the XML input file.
     *
     */
    public POIParser(String filename) {
        boolean validation = false;
        boolean ignoreWhitespace = true;
        boolean ignoreComments = false;
        boolean putCDATAIntoText = false;
        boolean createEntityRefs = false;

        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        dbf.setValidating(validation);
        dbf.setIgnoringComments(ignoreComments);
        dbf.setIgnoringElementContentWhitespace(ignoreWhitespace);
        dbf.setCoalescing(putCDATAIntoText);
        dbf.setExpandEntityReferences(!createEntityRefs);
        try {
            db = dbf.newDocumentBuilder();
            Document doc = null;
            doc = db.parse(new File(DIRECTORY + filename));
            parse(doc);
        }catch(Exception e){
            System.out.println("Error openning input document: " +filename + " using system defaults");

        }

    } //Constructor



    /* ===================================================================== */
    /** This method takes in an Element and returns the concatenation of all
     *  of its text subnodes.
     *
     *  @param  e   The Element to analize
     *  @return the text from all the direct subnodes of node
     *
     */
    public String getText(Element e){
        String result = "";
        for (Node n = e.getFirstChild(); n != null; n = n.getNextSibling())
            try {
                result += ((CharacterData)n).getData();
            } catch (ClassCastException cce) {} //not a text node
        return result;
    }// getText




    /* ===================================================================== */
    /** This method takes in a Document node, which heads the XML tree
     *  structure.  The method will search through all the nodes for certain
     *  named nodes which contain values we are after.
     *
     *  @param  node   The root node for the XML Document.
     *
     */
    public void parse(Document doc){

        Position pos;

        for (Node n = doc.getDocumentElement().getFirstChild(); n != null;  n = n.getNextSibling()) {
            try {

                Element e = (Element)n;
                String nodeName = e.getTagName();        //the node's name
                System.out.println("checking for the " + nodeName + " value");

              	if (nodeName.equals("building")) {

					pos = new Position();

					double lat = ((int)(Double.parseDouble(e.getAttribute("latitude"))/100))*60;
					lat += (Double.parseDouble(e.getAttribute("latitude")) - (((int)(Double.parseDouble(e.getAttribute("latitude"))/100))*100));
					pos.setLatitude(lat);

					double lng = ((int)(Double.parseDouble(e.getAttribute("longitude"))/100))*60;
					lng += (Double.parseDouble(e.getAttribute("longitude")) - (((int)(Double.parseDouble(e.getAttribute("longitude"))/100))*100));
					lng = lng * -1;
					pos.setLongitude(lng);

                    pois.addElement(new Building(pos , e.getAttribute("name") ));

              	}else if (nodeName.equals("road")) {
					pos = new Position();

					double lat = ((int)(Double.parseDouble(e.getAttribute("latitude"))/100))*60;
					lat += (Double.parseDouble(e.getAttribute("latitude")) - (((int)(Double.parseDouble(e.getAttribute("latitude"))/100))*100));
					pos.setLatitude(lat);

					double lng = ((int)(Double.parseDouble(e.getAttribute("longitude"))/100))*60;
					lng += (Double.parseDouble(e.getAttribute("longitude")) - (((int)(Double.parseDouble(e.getAttribute("longitude"))/100))*100));
					lng = lng * -1;
					pos.setLongitude(lng);

                    pois.addElement(new Road(pos , e.getAttribute("name") ));

              	}else if (nodeName.equals("path")) {
					pos = new Position();

					double lat = ((int)(Double.parseDouble(e.getAttribute("latitude"))/100))*60;
					lat += (Double.parseDouble(e.getAttribute("latitude")) - (((int)(Double.parseDouble(e.getAttribute("latitude"))/100))*100));
					pos.setLatitude(lat);

					double lng = ((int)(Double.parseDouble(e.getAttribute("longitude"))/100))*60;
					lng += (Double.parseDouble(e.getAttribute("longitude")) - (((int)(Double.parseDouble(e.getAttribute("longitude"))/100))*100));
					lng = lng * -1;
					pos.setLongitude(lng);

                    pois.addElement(new Path(pos , e.getAttribute("name") ));

                } //if


            } catch (ClassCastException cce) {     //node was not an element
            } catch (NumberFormatException nfe) {  //numberic value wasn't there

            }//endtry

        }//endfor

    } //endmethod parse




    /* ===================================================================== */
    /** This method is for getting the ref points of the map.
     *
     *  @return  Vector   A set of at least two reference points.
     *
     */
    public Vector getPOIs(){return pois;}




} //class: POIParser
